import RegFile::*;
import RWire::*;
import ProcTypes::*;

`include "asim/provides/low_level_platform_interface.bsh"
`include "asim/provides/soft_connections.bsh"
`include "asim/provides/fpga_components.bsh"
`include "asim/provides/common_services.bsh"

//-----------------------------------------------------------
// Register file module
//-----------------------------------------------------------

interface BRegFile;
   method Action   wr( Rindx rindx, Bit#(32) data );
   method ActionValue#( Bit#(32)) rd1( Rindx rindx );
   method ActionValue#( Bit#(32)) rd2( Rindx rindx );
endinterface

(* doc = "synthesis attribute ram_style mkBRegFile distributed;" *)
(* synthesize *)
module mkBRegFile(BRegFile);

   LUTRAM#(Rindx, Bit#(32)) rf <- mkLUTRAMU_RegFile();
   RWire#(Tuple2#(Rindx, Bit#(32))) rw <-mkRWire();

   method Action wr( Rindx rindx, Bit#(32) data );
      rf.upd( rindx, data );
      rw.wset(tuple2(rindx,data));
   endmethod

   method ActionValue#(Bit#(32)) rd1 (Rindx r);
      if (r == 0) return 0;
      else begin
            case (rw.wget()) matches
	     tagged Valid {.wr, .d} :
	       return (wr == r) ? d : rf.sub(r);
	     tagged Invalid : return rf.sub(r);
     	    endcase
	   end
   endmethod

   method ActionValue#(Bit#(32)) rd2 (Rindx r);
      if (r == 0) return 0;
      else begin
            case (rw.wget()) matches
	     tagged Valid {.wr, .d} :
	       return (wr == r) ? d : rf.sub(r);
	     tagged Invalid : return rf.sub(r);
     	    endcase
	   end
   endmethod
   
endmodule
